import { hoverTooltip } from '@codemirror/tooltip';
import { ViewPlugin } from '@codemirror/view';
import { addEvents } from "../../../helpers/add-events.js";
import controlledPromise from "../../../helpers/controlled-promise.js";
import { defineEffectField } from "../../../helpers/define-effect-field.js";
import { renderPartsTo } from "../../../helpers/render-parts.js";
const [lastInfotipRequest, dispatchLastInfotipRequestChanged] = defineEffectField(null);
export function kindsToClassNames(kinds) {
  return kinds.map(kind => 'mirrorsharp-infotip-icon-' + kind);
}

function renderInfotip({
  sections,
  kinds
}) {
  const wrapper = document.createElement('div');
  sections.forEach((section, index) => {
    const element = document.createElement('div');
    element.className = 'mirrorsharp-parts-section';

    if (index === 0) {
      const icon = document.createElement('span');
      icon.classList.add('mirrorsharp-infotip-icon', ...kindsToClassNames(kinds));
      element.appendChild(icon);
    }

    renderPartsTo(element, section.parts);
    wrapper.appendChild(element);
  });
  return wrapper;
}

export const infotipsFromServer = connection => {
  const requestInfotip = (view, pos) => {
    const infotipPromise = controlledPromise();
    dispatchLastInfotipRequestChanged(view, {
      pos,
      resolve: infotipPromise.resolve
    }); // eslint-disable-next-line @typescript-eslint/no-floating-promises

    connection.sendRequestInfoTip(pos);
    return infotipPromise.promise;
  };

  const receiveInfotipFromServer = ViewPlugin.define(view => {
    const removeEvents = addEvents(connection, {
      message: message => {
        if (message.type !== 'infotip') return;
        const request = view.state.field(lastInfotipRequest);
        if (!request || !message.sections) return;
        const {
          span
        } = message;
        if (request.pos < span.start || request.pos > span.start + span.length) return;
        request.resolve({
          pos: request.pos,
          end: span.start + span.length,
          create: () => ({
            dom: renderInfotip(message)
          }),
          class: 'mirrorsharp-infotip'
        });
      }
    });
    return {
      destroy: removeEvents
    };
  });
  return [lastInfotipRequest, hoverTooltip(requestInfotip), receiveInfotipFromServer];
};